<?php

namespace Drupal\xray_audit\Plugin\tasks\DisplaysGroup;

use Drupal\xray_audit\Plugin\XrayAuditTaskPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\xray_audit\Services\DisplayModesEntityArchitectureInterface;

/**
 * Plugin implementation of queries_data_node.
 *
 * @XrayAuditTaskPlugin (
 *   id = "ux_ui_architecture",
 *   label = @Translation("Data architecture"),
 *   description = @Translation("Data architecture and display modes."),
 *   group = "displays",
 *   sort = 1,
 *   operations = {
 *      "data_architecture_node" = {
 *          "label" = "Data architecture [Nodes]",
 *          "description" = "Data architecture and display modes of Nodes.",
 *          "dependencies" = {"node"}
 *       },
 *     "data_architecture_paragraphs" = {
 *          "label" = "Data architecture [Paragraphs]",
 *          "description" = "Data architecture and display modes of
 *   Paragraphs.",
 *          "dependencies" = {"paragraphs"}
 *      },
 *     "data_architecture_media" = {
 *          "label" = "Data architecture [Medias]",
 *          "description" = "Data architecture and display modes of Medias.",
 *          "dependencies" = {"media"}
 *     },
 *     "data_architecture_taxonomies" = {
 *          "label" = "Data architecture [Taxonomies]",
 *          "description" = "Data architecture and display modes of
 *   Taxonomies.",
 *          "dependencies" = {"taxonomy"}
 *      }
 *   },
 *   dependencies = {"field"}
 * )
 */
class XrayAuditEntityDisplaysPlugin extends XrayAuditTaskPluginBase {

  /**
   * Entity type manager.
   *
   * @var \Drupal\xray_audit\Services\DisplayModesEntityArchitectureInterface
   */
  protected $extractorDisplayModes;

  /**
   * Constructs a \Drupal\Component\Plugin\PluginBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\xray_audit\Services\DisplayModesEntityArchitectureInterface $extractorDisplayModes
   *   Entity type manager.
   */
  public function __construct(array $configuration, string $plugin_id, $plugin_definition, DisplayModesEntityArchitectureInterface $extractorDisplayModes) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->extractorDisplayModes = $extractorDisplayModes;
  }

  /**
   * Create static method.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   Container.
   * @param array $configuration
   *   Configuration.
   * @param string $plugin_id
   *   Plugin id.
   * @param mixed $plugin_definition
   *   Plugin definition.
   *
   * @return static
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('xray_audit.extractor_display_modes_entity'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDataOperationResult(string $operation = NULL) {
    switch ($operation) {
      case 'data_architecture_node':
        return $this->extractorDisplayModes->getData('node_type', 'node');

      case 'data_architecture_paragraphs':
        return $this->extractorDisplayModes->getData('paragraphs_type', 'paragraph');

      case 'data_architecture_media':
        return $this->extractorDisplayModes->getData('media_type', 'media');

      case 'data_architecture_taxonomies':
        return $this->extractorDisplayModes->getData('taxonomy_vocabulary', 'taxonomy_term');
    }

    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function buildDataPresentationForCli(array $data, string $operation = NULL) {
    // @todo Implement buildDataPresentationForCli() method.
    return '';
  }

  /**
   * {@inheritdoc}
   */
  public function buildDataRenderArray(array $data, string $operation = NULL) {
    $description = '';
    $operation_data = $this->getOperations()[$operation] ?? NULL;
    if ($operation_data) {
      $description = $operation_data['description'] ?? '';
    }

    $header = $data['header_table'] ?? [];
    $rows = [];

    foreach ($data['results_table'] ?? [] as $delta => $row) {
      $rows[$delta] = [
        'data' => $row,
        'class' => ['xray-audit__row'],
      ];
    }

    $build = [];

    if ($description) {
      $build['#markup'] = '<p>' . $description . '</p>';
    }

    $build['summary_table_introduction'] = [
      '#markup' => "<p></p><h2>Data architecture overview</h2><p></p>",
      '#weight' => '0',
    ];
    $build['summary_table'] = $this->getResumTable($rows);

    $build['summary_table_displays'] = $this->getResumeTableDisplays($rows);

    $build['main_table_introduction'] = [
      '#markup' => "<p></p><h2>Data architecture</h2>",
      '#weight' => 23,
    ];

    if (isset($data['computed'])) {
      $build['computed'] = [
        '#markup' => "<p></p><b>Computed fields:</b> {$data['computed']}</p>",
        '#weight' => 25,
      ];
    }

    $build['main_table'] = [
      '#theme' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#weight' => 30,
      '#attributes' => [
        'class' => 'xray-audit__table',
      ],
      '#attached' => [
        'library' => [
          'xray_audit/xray_audit',
        ],
      ],
    ];

    return $build;
  }

  /**
   * Build the render array for resume.
   *
   * @param array $rows
   *   Data to build the table.
   *
   * @return array
   *   Render array.
   */
  protected function getResumTable(array $rows) {
    $rows_current_data = [];
    $total = 0;

    foreach ($rows as $k_row => $row) {
      $row = $row['data'] ?? $row;
      $rows_current_data[$k_row]['type'] = $row['type'];
      $rows_current_data[$k_row]['displays'] = '';
      $rows_current_data[$k_row]['count'] = 0;
      foreach ($row as $key => $content) {
        if (in_array($key, ['type', 'fields']) || empty($content)) {
          continue;
        }
        if ($rows_current_data[$k_row]['count'] > 0) {
          $rows_current_data[$k_row]['displays'] .= ', ';
        }
        $rows_current_data[$k_row]['displays'] .= $key;
        $rows_current_data[$k_row]['count']++;
        $total++;
      }
    }

    $rows_current_data[] = [
      'type' => 'TOTAL',
      'displays' => '',
      'count' => $total,
    ];

    return [
      '#theme' => 'table',
      '#header' => ['Bundle', 'Displays', 'Displays Count'],
      '#rows' => $rows_current_data,
      '#weight' => 20,
    ];
  }

  /**
   * Build the render array for resume (displays).
   *
   * @param array $rows
   *   Data to build the table.
   *
   * @return array
   *   Render array.
   */
  protected function getResumeTableDisplays(array $rows) {
    $rows_current_data = [];
    $total = 0;

    foreach ($rows as $row) {
      $row = $row['data'] ?? $row;
      $type = $row['type'] ? (string) $row['type'] : '';
      $type = strip_tags($type);
      unset($row['type']);
      unset($row['fields']);

      foreach ($row as $key => $content) {

        if (empty($content)) {
          continue;
        }

        // Display key.
        $rows_current_data[$key]['display'] = $key;

        // Entity.
        if (!isset($rows_current_data[$key]['types'])) {
          $rows_current_data[$key]['types'] = $type;
        }
        else {
          $rows_current_data[$key]['types'] .= ", $type";
        }

        // Display counts.
        if (!isset($rows_current_data[$key]['count'])) {
          $rows_current_data[$key]['count'] = 1;
        }
        else {
          $rows_current_data[$key]['count']++;
        }
        $total++;
      }

    }

    $rows_current_data['total'] = [
      'display' => 'TOTAL',
      'types' => '',
      'count' => $total,
    ];

    return [
      '#theme' => 'table',
      '#header' => ['Displays', 'Bundles', 'Bundles Count'],
      '#rows' => $rows_current_data,
      '#weight' => 21,
    ];

  }

}
