<?php

namespace Drupal\xray_audit\Plugin\tasks\QueryGroup;

use Drupal\user\Entity\Role;

/**
 * Plugin implementation of queries_data_node.
 *
 * @XrayAuditTaskPlugin (
 *   id = "queries_data_user",
 *   label = @Translation("Data about User and Roles"),
 *   description = @Translation("Queries execute on database to get reports about user."),
 *   group = "queries_data",
 *   sort = 3,
 *   operations = {
 *      "count_user" = {
 *          "label" = "User count",
 *          "description" = "Number of user."
 *       },
 *       "description_of_roles" = {
 *          "label" = "Roles",
 *          "description" = "Roles"
 *       },
 *      "count_user_per_role" = {
 *          "label" = "User count per roles",
 *          "description" = "User count per roles."
 *       },
 *       "permissions_roles" = {
 *          "label" = "Permissions per role",
 *          "description" = "Permissions per role."
 *       }
 *    },
 *   dependencies = {"user"}
 * )
 */
class XrayAuditQueryTaskUserPlugin extends XrayAuditQueryTaskPluginBase {

  /**
   * {@inheritdoc}
   */
  public function getDataOperationResult(string $operation = NULL) {
    switch ($operation) {
      case 'count_user':
        return $this->countUser();

      case 'count_user_per_role':
        return $this->usersPerRole();

      case 'description_of_roles':
        return $this->rolesDescription();

      case 'permissions_roles':
        return $this->permissionsPerRole();
    }
    return [];
  }

  /**
   * Get data for operation "count_user".
   *
   * @return array
   *   Data.
   */
  protected function countUser() {

    $headerTable = [$this->t('Number of users')];
    $resultTable = [];

    $storage = $this->entityTypeManager->getStorage('user');

    $resultTable[] = [$storage->getQuery()->accessCheck(FALSE)->count()->execute()];

    return [
      'header_table' => $headerTable,
      'results_table' => $resultTable,
    ];

  }

  /**
   * Users per role.
   *
   * @return array
   *   Render array.
   */
  public function usersPerRole() {

    $headerTable = [$this->t('roles'), $this->t('count')];
    $resultTable = [];

    $query = $this->database->query("select roles_target_id as role, count(1) as count from user__roles group by role");
    $result = $query->fetchAll();

    foreach ($result as $row) {
      $resultTable[] = [$row->role, $row->count];
    }

    return [
      'header_table' => $headerTable,
      'results_table' => $resultTable,
    ];

  }

  /**
   * Description roles.
   *
   * @return array
   *   Render array.
   */
  public function rolesDescription() {
    $headerTable = ['id', 'label', 'langcode'];
    $resultTable = [];
    $roles = Role::loadMultiple();
    foreach ($roles as $role) {
      $resultTable[] = [
        $role->get('id'),
        $role->get('label'),
        $role->get('langcode'),
      ];
    }
    return [
      'header_table' => $headerTable,
      'results_table' => $resultTable,
    ];
  }

  /**
   * Permissions and roles.
   *
   * @return array
   *   Render array.
   */
  public function permissionsPerRole() {
    $headerTable = ['role', 'permissions'];
    $resultTable = [];
    $roles = Role::loadMultiple();
    foreach ($roles as $role) {
      $resultTable[] = [
        $role->get('label'),
        implode(', ', $role->get('permissions')),
      ];
    }
    return [
      'header_table' => $headerTable,
      'results_table' => $resultTable,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildDataPresentationForCli(array $data, string $operation = NULL) {
    // @todo Implement buildDataPresentationForCli() method.
    return '';
  }

}
